#!/bin/bash -e

# Get defaults from environment variables
#get repo path
REPO=${REPO:-/build}
export REPO
#get doc path
DOC_DIR=${DOC_DIR:-doc}
#get output path
OUTPUT_DIR=${OUTPUT_DIR:-$REPO/$DOC_DIR/_build}

#read the title/version from the settings file
# right now this has defaults if the settings file is missing a key or if missing entirely
# TODO: decide if this is the right behaviour
# doc title
if [ -z "$TITLE" ]; then
    TITLE=$(jq -r '.title // "TMP"' $REPO/settings.json || echo "TMP")
fi
# project version
if [ -z "$VERSION" ]; then
    VERSION=$(jq -r '.version // "TBD"' $REPO/settings.json || echo "TBD")
fi
export DOXYGEN_PROJECT=$TITLE
export DOXYGEN_OUTPUT="$OUTPUT_DIR/$VERSION/_doxygen"
export DOXYGEN_INPUT

HTML_OUTPUT_DIR=$OUTPUT_DIR/$VERSION/html
PDF_OUTPUT_DIR=$OUTPUT_DIR/$VERSION/pdf
DOCTREES_PATH=$OUTPUT_DIR/$VERSION/.doctrees
TEMPLATE_DIR=$REPO/$DOC_DIR/_templates


# Do some checks that the container has been run as suggested
echo "Windows users are recommended to run under wsl2 to make mounting and permissions easier and more performant"

if [ ! -d $REPO/$DOC_DIR ]; then
    echo "== XMOS docs is not initialized =="
    echo " - Bind mount a repo with a \"$DOC_DIR\" folder to $REPO to work on existing documentation"
    echo " >    docker run -t -v \$REPO_PATH:$REPO -u \"\$(id -u):\$(id -g)\" doc_builder"
    exit 1
fi

if [ "$EUID" -eq 0 ]; then
    echo "== XMOS docs should not be run as default user =="
    echo " - Pass the container your user info"
    echo " >    docker run -t -v \$REPO_PATH:$REPO -u \"\$(id -u):\$(id -g)\" doc_builder"
    exit 1
fi

cd $REPO/$DOC_DIR

#removes old content if this destination folder (i.e. named with this version) already exists
#and creates new empty folders
if [ -d $OUTPUT_DIR/$VERSION ]; then
    echo "Removing old content in output directory: $VERSION"
    rm -rf $OUTPUT_DIR/$VERSION/*
fi
mkdir -p $OUTPUT_DIR

if [ -d $TEMPLATE_DIR ]; then
    echo "Removing old templates directory"
    rm -rf $TEMPLATE_DIR
fi
mkdir -p $TEMPLATE_DIR


#create the html template which provides the version drop-down
echo "Building version dropdown template"
python /docs/build_version_drop-down.py -l $VERSION -p $OUTPUT_DIR -o $TEMPLATE_DIR/versioning.html

#create the architecture_tools_info rst page
echo "Building architecture/tools page"
python /docs/build_architecture_and_tools_page.py -p $REPO -o  $TEMPLATE_DIR/architecture_tools_info.rst

#run doxygen
if [ ! -z "$DOXYGEN" ] || [ ! -z "$DOXYGEN_INPUT" ] || [ ! -z "$DOXYGEN_INCLUDE" ]; then
    if [ -z "$DOXYGEN_INPUT" ]; then
        #caller did no specify Doxygen inputs, so glob for them
        DOXYGEN_INPUT=$(shopt -s globstar && ls -d1 $REPO/**/api $REPO/**/src | tr '\n' ' ')
    fi

    if [ -d $DOXYGEN_OUTPUT ]; then
        echo "Removing old doxygen output directory"
        rm -rf $DOXYGEN_OUTPUT
    fi
    mkdir -p $DOXYGEN_OUTPUT

    doxygen /docs/Doxyfile
fi

#generate html
mkdir -p $HTML_OUTPUT_DIR

#check for top-level index.rst
if [ -f "$REPO/index.rst" ]; then
    SPHINX_SOURCE_DIR=$REPO
else
    #assume index.rst is in the root of the doc dir
    SPHINX_SOURCE_DIR="$REPO/$DOC_DIR"
fi

sphinx-build -b html -d $DOCTREES_PATH -c /docs $SPHINX_SOURCE_DIR $HTML_OUTPUT_DIR
sphinx-build -b linkcheck -c /docs $SPHINX_SOURCE_DIR $HTML_OUTPUT_DIR

#generate pdf
if [ ! -z "$PDF" ]; then
    mkdir -p $PDF_OUTPUT_DIR
    sphinx-build -b latex -d $DOCTREES_PATH -c /docs $REPO $PDF_OUTPUT_DIR
    pushd $PDF_OUTPUT_DIR &> /dev/null
    make LATEXMKOPTS="-f -xelatex -interaction=batchmode"
    # # remove all non-pdf files
    if [ -z "$DEV" ]; then
        find . ! -name '*.pdf' -delete
    fi
    popd &> /dev/null
fi

#remove sphinx cache and templates
if [ -z "$DEV" ]; then
    rm -rf $DOCTREES_PATH
    rm -rf $TEMPLATE_DIR
fi

#copies the new version to "latest" as this allows old versions always to navigate to the newest one
LATEST_VERSION=${LATEST_VERSION:-$VERSION}
if [ -d $OUTPUT_DIR/latest ]; then
    echo "Replace latest content with a copy of $LATEST_VERSION"
    rm -rf $OUTPUT_DIR/latest
fi
mkdir -p $OUTPUT_DIR/latest
#dealing with latest version override
cp -r $OUTPUT_DIR/$LATEST_VERSION/* $OUTPUT_DIR/latest
