import os
from mako.template import Template
import argparse
import json

def build_template(thisVersion, destFolder, outputFile):

    # Description
    #
    # This program takes either a list of strings or a directory as an argument.  In the case of a directory it simply extracts a list of
    # immediate sub-directories, to use in place of the list of strings.
    # The purpose is that we store versions of the html documentation in different folders and each sub-dir is a containing folder.
    # We use mako to complete a template to generate a new html file to include in the Sphinx build (included in the conf.py file)
    # This template includes the javascript that navigates through the versions.
    # outputFileName is the resulting path/file of the versioning.html file.  Note that this appears in conf.py, so don't change it unless
    # you do so in both places!

    # Sample usage
    #build_template(versions=("1","2"))
    #build_template(destFolder="C:/Users/andrewdewhurst/Documents/temp/Explore_HW_Manual/html")


    # -- Define the template ---------------------------------------------------------
    raw_template = """
<select name="version" id="version-select" value="Latest" onchange="changeVersion(this.value)" >
<option selected disabled>Choose version</option>
%for version in versions:
    <option value="/${root}/${version}">${version}</option>
%endfor
</select>

<script>
function changeVersion(val) {
    var loc = window.location.toString();
    var docroot = "/${root}/"
    var i = loc.lastIndexOf(docroot);
    if (i >= 0) {
        var s1 = loc.substring(0, i);
        var i2 = loc.indexOf("/", i+docroot.length);
        var s2 = loc.substring(i2);
        nl=s1 + val + s2;

        var request = new XMLHttpRequest();  
        request.open('GET', nl, true);
        request.onreadystatechange = function() {
            if (request.readyState === 4) {
                if (request.status === 404) {  
                    alert("Oh no, it does not exist!");
                    window.location.assign(loc);
                    location.reload();
                }
                else {
                    window.location.assign(nl);
                }
            }
        };
        request.send();
    }
}
</script>"""

    # -- Now fill in the template to create a new html file ---------------------------------------------------------
    #Check if a list is provided, and always append "latest" so any version can navigate to the latest future version
    versions = []
    for f in sorted(os.listdir(destFolder)):
        if f != "latest" and f != thisVersion:  # skip latest in folder detection
            if os.path.isdir(destFolder + "/" + f):
                versions.append(f)
    versions.append(thisVersion)  # add this version explicitly
    versions.append("latest")  # add latest shortcut

    #Use mako to process the template
    mytemplate = Template(raw_template)
    root = os.path.basename(os.path.normpath(destFolder))
    filledTemplate = mytemplate.render(root=root, versions=(versions))

    #Create a file
    f= open(outputFile,"w+")

    #Write template data to the file created
    f.write(filledTemplate)

    #Close file
    f.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-l", help='This latest version number', required=True)
    parser.add_argument("-o", default='./_templates/versioning.html', help='optional path and filename of the resulting html file.')
    parser.add_argument("-p", help='path to html output folder')
    args = parser.parse_args()
    build_template(thisVersion=args.l, destFolder=args.p, outputFile=args.o)
